/*
 * ioDriver.c
 * 
 * This software is provided as-is.  Lantronix and the author disclaim to the fullest 
 * extent authorized by law any and all warranties, whether express or implied, 
 * including, without limitation, any implied warranties of title, non-infringement, 
 * merchantability or fitness for a particular purpose.   Neither the author nor 
 * Lantronix are responsible for any damage or losses of any kind caused by the use or 
 * misuse of this software.  IN NO EVENT SHALL THE AUTHOR OR LANTRONIX BE 
 * LIABLE FOR ANY DAMAGES, INCLUDING  INCIDENTAL OR CONSEQUENTIAL 
 * DAMAGES, IN CONNECTION WITH OR ARISING FROM THE FURNISHING, 
 * PERFORMANCE, OR USE OF THIS SOFTWARE.
 *
 */

#include <mqx.h>
#include <bsp.h>
#include <serial.h>
#include <adc.h>
#include "main.h"

MQX_FILE_PTR InitializeIO(void);
void SetOutput(int signal, boolean state);
void ResetOutputs(void);
void ToggleLED(int);
void SetupLED(LWGPIO_STRUCT, int, int);
void SendStatus(MQX_FILE_PTR);
void getXSR(MQX_FILE_PTR, char[]);
void SendPost(MQX_FILE_PTR, char[], int);
int waitPostResponse(MQX_FILE_PTR);
ADC_VALUE_TYPE getADC(void);
void waitForPrompt(MQX_FILE_PTR);
void receiveLine(MQX_FILE_PTR, char[]);
void assignToken(char[], char[], int size);

static char ipAdd[15];
static char ssid[20];
static char rssi[12];
char uid[15] = "00000000000000";
char LED_Status[4];
MQX_FILE_PTR pot;

/* ADC device init struct */
const ADC_INIT_STRUCT adc_init = {
    ADC_RESOLUTION_DEFAULT,     /* resolution */
};

/* Logical channel #1 init struct */
const ADC_INIT_CHANNEL_STRUCT adc_channel_param1 = 
{
    ADC1_SOURCE_ADPM1, /* physical ADC channel */
    ADC_CHANNEL_MEASURE_LOOP | ADC_CHANNEL_START_NOW, /* runs continuously after open */
    1,             /* number of samples in one run sequence */
    0,              /* time offset from trigger point in us */
    100000,         /* period in us (= 0.1 sec) */
    0x10000,        /* scale range of result (not used now) */
    1,             /* circular buffer size (sample count) */
    ADC_PDB_TRIGGER,     /* logical trigger ID that starts this ADC channel */
};



MQX_FILE_PTR InitializeIO(void)
{
	LWGPIO_STRUCT led1, led2, led3, led4;

	SetupLED(led1,BSP_LED1,BSP_LED1_MUX_GPIO);
	SetupLED(led2,BSP_LED2,BSP_LED2_MUX_GPIO);
	SetupLED(led3,BSP_LED3,BSP_LED3_MUX_GPIO);
	SetupLED(led4,BSP_LED4,BSP_LED4_MUX_GPIO);
	ResetOutputs();
	
	fopen("adc1:", (const char*)&adc_init);
	pot = fopen("adc1:" "first", (const char*)&adc_channel_param1);
	
	return fopen("ittyd:", NULL);
	
}


void SetupLED(LWGPIO_STRUCT led, int bsp_led, int bsp_led_mux_gpio)
{
	if (!lwgpio_init(&led, bsp_led, LWGPIO_DIR_OUTPUT, LWGPIO_VALUE_HIGH))
	{
		printf("Initializing LED GPIO as output failed.\n");
	}
	else
	{
		lwgpio_set_functionality(&led,bsp_led_mux_gpio);
	}
}

void SetOutput(int signal, boolean state) 
{
	
	 switch (signal) {
	 	 case 1:
	 		 lwgpio_set_pin_output(BSP_LED1, (state) ? LWGPIO_VALUE_LOW : LWGPIO_VALUE_HIGH);
	 		 break;
	 	case 2:
	 		 lwgpio_set_pin_output(BSP_LED2, (state) ? LWGPIO_VALUE_LOW : LWGPIO_VALUE_HIGH);
	 		 break;
	 	case 3:
	 		 lwgpio_set_pin_output(BSP_LED3, (state) ? LWGPIO_VALUE_LOW : LWGPIO_VALUE_HIGH);
	 		 break;
	 	case 4:
	 		 lwgpio_set_pin_output(BSP_LED4, (state) ? LWGPIO_VALUE_LOW : LWGPIO_VALUE_HIGH);
	 		 break;
	 	 }
}

void ResetOutputs(void) 
{
	uint_32 i;
	for (i=1;i<=4;i++) 
	{
		SetOutput(i,FALSE);
		LED_Status[i-1] = '0';
	}
}

void ToggleLED(int signal)
{
	switch (signal) {
	case 1:
		lwgpio_toggle_pin_output(BSP_LED1);
		LED_Status[0] = LED_Status[0]^1;
		break;
	case 2:
		lwgpio_toggle_pin_output(BSP_LED2);
		LED_Status[1] = LED_Status[1]^1;
		break;
	case 3:
		lwgpio_toggle_pin_output(BSP_LED3);
		LED_Status[2] = LED_Status[2]^1;
		break;
	case 4:
		lwgpio_toggle_pin_output(BSP_LED4);
		LED_Status[3] = LED_Status[3]^1;
		break;
	}
}

ADC_VALUE_TYPE getADC()
{
	ADC_RESULT_STRUCT data;
	while(!read(pot,&data, sizeof(data)))
	{
		_time_delay(100);
	}
	return data.result;
}

/*
 * This creates the Status message from the global variables
 * that are populated by getting the status of the xPico Wi-Fi.
 */
void SendStatus(MQX_FILE_PTR ser_dev)
{
	char ctrlA = 0x1;
	
	write(ser_dev,LED_Status,4);
	write(ser_dev,"\n", 1);
	write(ser_dev,ssid, sizeof(ssid));
	write(ser_dev,"\n", 1);
	write(ser_dev,ipAdd, sizeof(ipAdd));
	write(ser_dev,"\n", 1);
	write(ser_dev,rssi, sizeof(rssi));
	write(ser_dev, &ctrlA, 1);
	
}

/*
 * This is used to send the HTTP POST. Note that it starts with 
 * the Ctrl-B character, which is configured in the xPico Wi-Fi
 * as the "Start Character" in the CONNECT menu.
 */
void SendPost(MQX_FILE_PTR ser_dev, char buffer[], int size)
{
	char ctrlB = 0x2;
	write(ser_dev,&ctrlB,1);
	_time_delay(500);
	write(ser_dev, buffer, size);
}

/*
 * This will wait for a response from the Google App Engine when
 * doing the HTTP POST. After the response is received, it will send
 * a Ctrl-A character on the serial port. This is configured in the
 * xPico Wi-Fi as the "Disconnect Character" in the DISCONNECT menu.
 */
int waitPostResponse(MQX_FILE_PTR ser_dev)
{
	int timeout = 0;
	int received = 0;
	char c;
	char ctrlA = 0x1;
	
	while(TRUE)
	{
		if (fstatus(ser_dev))
		{
			timeout = 0;
			received = 1;
			c = fgetc( ser_dev );
			//printf("%c", c);
		} else {
			if ( (received == 0 && timeout > 1000) || (received == 1 && timeout > 100)) {
				break;
			} else {
				timeout += 1;
				_time_delay(20);
			}
		}
	}
	write(ser_dev,&ctrlA,1);
	return received;
}

/*
 * Function to connect on the second serial port of the xPico Wi-Fi
 * and retrieve the current status (XSR).
 * 
 * This function parses the Command Line Interface (CLI) to get to the
 * XSR status. It receives the XML, and then parses the XML to extract
 * the current SSID, RSSI, and IP address of the wlan0 interface. It then
 * populates the global variables with those values for status reporting.
 */
void getXSR(MQX_FILE_PTR conf_dev, char group[])
{
	char buffer[200];
	char *value;
	char *next = "NONE";
	char c;
	_mqx_int size;
	
	write(conf_dev, "\r", 1 );
	size = snprintf(buffer, 200, "xml\r");
	write(conf_dev, buffer, size);
	waitForPrompt(conf_dev);
	_time_delay(1000);
	size = snprintf(buffer, 200, "xsr dump %s\r", group);
	write(conf_dev, buffer, size);

	while(TRUE)
	{

		receiveLine(conf_dev, buffer);
		if (buffer[0] == '\0') break;
		//printf("%s", buffer);
		if (strstr(buffer, "/statusrecord") != NULL) {
			size = snprintf(buffer, 200, "exit\r");
			write(conf_dev, buffer, size);
			_time_delay(100);
			write(conf_dev, buffer, size);
			_time_delay(100);
			fflush(conf_dev);
			break;
		} else if (strstr(buffer, "SSID") != NULL && strstr(buffer, "BSSID") == NULL){
			snprintf(buffer, 200, "");
			receiveLine(conf_dev, buffer);
			assignToken(buffer, ssid, 20);
		} else if (strstr(buffer, "RSSI") != NULL) {
			snprintf(buffer, 200, "");
			receiveLine(conf_dev, buffer);
			assignToken(buffer, rssi, 12);
		} else if (strstr(buffer, "IP Address") != NULL) {
			snprintf(buffer, 200, "");
			receiveLine(conf_dev, buffer);
			assignToken(buffer, ipAdd, 15);
		} else if (strstr(buffer, "Serial Number") != NULL) {
			snprintf(buffer, 200, "");
			receiveLine(conf_dev, buffer);
			assignToken(buffer, uid, 15);
		}
		snprintf(buffer, 200, "");
	}
}

void waitForPrompt(MQX_FILE_PTR conf_dev)
{
	char c;
	while (TRUE)
	{
		if (fstatus(conf_dev)) {
			c = fgetc(conf_dev);
			if (c == '>') {
				break;
			}
		} else {
			_time_delay(100);
		}
	}
}

void assignToken(char buffer[], char variable[], int size)
{
	strtok(buffer, "><");
	strtok(NULL, "><");
	strncpy(variable, strtok(NULL,"><"), size);
}

void receiveLine(MQX_FILE_PTR dev, char buffer[]) 
{
	char c;
	int timeout = 0;
	
	while (TRUE)
	{
		if (fstatus(dev)) {
			timeout = 0;
			c = fgetc(dev);
			snprintf(buffer, 200, "%s%c",buffer, c);
			if (c == '\r') break;
		} else {
			timeout++;
			if (timeout > 100) break;
			_time_delay(50);
		}
	}
}
